<?php

namespace WebsiteBundle\Controller\Admin;

use Bazookas\AdminBundle\Controller\Base\BaseAdminListController;
use Bazookas\AdminBundle\PageBuilder\Base\BaseFormPageBuilder;
use Bazookas\AdminBundle\PageBuilder\Interfaces\ListPageBuilderInterface;
use Bazookas\AdminBundle\PageBuilder\ListPageBuilder;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Form\Admin\ExperimentAdminType;
use WebsiteBundle\PageBuilder\ExperimentFormPageBuilder;
use WebsiteBundle\Security\Roles;

class ExperimentsAdminController extends BaseAdminListController
{

  public function __construct() {
    parent::__construct();

    $this->builders[self::ACTION_ADD] = ExperimentFormPageBuilder::class;
  }

  protected function getEntityClass()
  {
    return Experiment::class;
  }

  protected function modifyListBuilder(Request $request, ListPageBuilderInterface $builder)
  {
    /**
     * @var ListPageBuilder $builders
     */
    $builder = parent::modifyListBuilder($request, $builder);
    $builder
      ->addField('name')
      ->addField('sessionCount')
      ->addField('participantCount')
      ->addField('activeParticipantCount')
      ->addField('researchers', [
        'template' => 'WebsiteBundle:List\Fields:researchersField.html.twig'
      ])
    ;

    //Add filters
    $builder
      ->addTextFilterField('name')
    ;

    return $builder;
  }

  protected function modifyAddBuilder(Request $request, BaseFormPageBuilder $builder) {
    $translator = $this->get('translator');
    $surveyBody = $translator->trans('admin.entities.experimentCondition.tooltips.survey.body', [
      '%URL%' => $this->generateUrl('survey_redirect', [], UrlGeneratorInterface::ABSOLUTE_URL)
    ], 'admin');

    $builder->setFormOptions([
      'surveyUrlToolTipBody' => $surveyBody
    ]);

    $builder = parent::modifyAddBuilder($request, $builder);

    return $builder;
  }

  protected function modifyEditBuilder(Request $request, BaseFormPageBuilder $builder) {
    $translator = $this->get('translator');
    $surveyBody = $translator->trans('admin.entities.experimentCondition.tooltips.survey.body', [
      '%URL%' => $this->generateUrl('survey_redirect', [], UrlGeneratorInterface::ABSOLUTE_URL)
    ], 'admin');

    $builder->setFormOptions([
      'surveyUrlToolTipBody' => $surveyBody
    ]);

    $builder = parent::modifyEditBuilder($request, $builder);

    return $builder;
  }

  protected function getFormClass($action) {
    switch($action) {
      case self::ACTION_ADD:
      case self::ACTION_EDIT:
        return ExperimentAdminType::class;
      default:
        return null;
    }
  }

  protected function hasAccess($action) {
    switch($action) {
      case self::ACTION_EDIT:
      case self::ACTION_ADD:
      case self::ACTION_REMOVE:
        return $this->isGranted(Roles::ROLE_ADMIN_RESEARCHER);
      case self::ACTION_LIST:
        return true;
      default:
        return false;
    }
  }

}
