<?php

namespace WebsiteBundle\Controller\Admin;

use Bazookas\AdminBundle\Controller\AdminController\BaseCMSUserAdminController;
use Bazookas\AdminBundle\Controller\Base\BaseAdminListController;
use Bazookas\AdminBundle\PageBuilder\Interfaces\ListPageBuilderInterface;
use Bazookas\AdminBundle\PageBuilder\ListPageBuilder;
use Symfony\Component\HttpFoundation\Request;
use WebsiteBundle\Entity\Researcher;
use WebsiteBundle\Form\Admin\ResearcherAdminType;

class ResearcherAdminController extends BaseCMSUserAdminController
{

  public function getEntityClass()
  {
    return Researcher::class;
  }

  protected function modifyListBuilder(Request $request, ListPageBuilderInterface $builder) {
    /**
     * @var ListPageBuilder $builder
     */
    $builder = BaseAdminListController::modifyListBuilder($request, $builder);

    $builder
      ->addBooleanField('enabled')
      ->addField('name')
      ->addField('surname')
      ->addField('email')
      ->addField('phoneNumber')
      ->addField('groups', [
        'template' => 'BazookasAdminBundle:List/Fields/CMSUser:_groups.html.twig'
      ])
      ->addField('experiments', [
        'template' => 'WebsiteBundle:List\Fields:experimentsField.html.twig'
      ])
    ;

    //Add filters
    $builder
      ->addTextFilterField('name')
      ->addTextFilterField('surname')
      ->addTextFilterField('email')
      ->addTextFilterField('groups.name', [
        'label' => 'admin.entities.researcher.fields.groups'
      ])
      ->addTextFilterField('experiments.name', [
        'label' => 'admin.entities.researcher.fields.experiments'
      ])
      ->addBooleanFilterField('enabled')
      ->setMaintainFilterState(true)
    ;

    return $builder;
  }

  protected function getFormClass($action) {
    switch($action) {
      case self::ACTION_ADD:
        return ResearcherAdminType::class;
      case self::ACTION_EDIT:
        return ResearcherAdminType::class;
      default:
        return null;
    }
  }
}
