<?php

namespace WebsiteBundle\Services\Data;

use Bazookas\APIFrameworkBundle\Exception\ApiException;
use Bazookas\APIFrameworkBundle\Services\Data\Base\BaseDataService;
use Doctrine\ORM\EntityManager;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\HttpFoundation\Response;
use WebsiteBundle\Entity\MoodAnswer;
use WebsiteBundle\Entity\TestSession;
use WebsiteBundle\Exception\ExceptionMap;

class MoodQuestionDataService extends BaseDataService
{
  /**
   * @var EntityManager
   */
  private $entityManager;

  public function __construct(EntityManager $entityManager) {
    $this->entityManager = $entityManager;
  }

  public function submitMoodQuestions(ParameterBag $params, ParameterBag $extraParams) {

    $participant = $extraParams->get('user');

    $sessionRepo = $this->entityManager->getRepository(TestSession::class);
    $session = $sessionRepo->getLastSessionWithoutMoodAnswers($participant, $participant->getExperiment());

    if (empty($session)) {
      throw new ApiException('Session not found.', ExceptionMap::SESSION_NO_SESSION, Response::HTTP_BAD_REQUEST);
    }

    $moodQuestions = $participant->getExperiment()->getMoodQuestions()->toArray();

    $data = $params->get('data');
    foreach($data as $item) {
      $question = null;
      foreach($moodQuestions as $question) {
        if ($question->getId() == $item['questionId']) {
          break;
        }
      }

      if (!empty($question)) {
        $answer = new MoodAnswer();
        $answer
          ->setParticipant($participant)
          ->setValue($item['answer'])
          ->setQuestion($question)
          ->setSession($session)
        ;

        $this->entityManager->persist($answer);
      }
    }

    $comment = $params->get('comment');
    if (!empty($comment)) {
      $session->setComment($comment);
      $this->entityManager->persist($session);
    }

    $this->entityManager->flush();

    //Need to return an empty response
    return [];
  }

}
