<?php

namespace WebsiteBundle\Services\Data;

use Bazookas\APIFrameworkBundle\Services\Data\Base\BaseDataService;
use Bazookas\CommonBundle\Exception\NotImplementedException;
use Doctrine\ORM\EntityManager;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\ExperimentCondition;
use WebsiteBundle\Entity\Participant;
use WebsiteBundle\Entity\TestSession;

class UserStatisticsService extends BaseDataService
{
  /**
   * @var EntityManager
   */
  private $entityManager;

  public function __construct(EntityManager $entityManager) {
    $this->entityManager = $entityManager;
  }

  public function getStatisticsData(Participant $participant) {
    $entityRepo = $this->entityManager->getRepository(TestSession::class);
    $sessions = $entityRepo->getSessions($participant);
    $experiment = $participant->getExperiment();

    $data = [];
    $data['accuracy'] = $this->getAccuracyData($sessions, $experiment);
    $data['speed'] = $this->getSpeedData($sessions, $experiment);

    if ($participant->getCondition()->getGamificationLevel() === ExperimentCondition::GAMIFICATION_HIGH) {
      $data['score'] = $this->getScoreData($sessions, $experiment);
    }

    return $data;
  }

  /**
   * @param TestSession[] $sessions
   * @return array
   */
  private function getAccuracyData(array $sessions, Experiment $experiment) {
    $data = [];

    foreach($sessions as $session) {
      if ($session['number'] !== 1 && $session['number'] !== $experiment->getTotalSessions()) {
        $data[] = round((($session['correctCount'] / $session['trialCount']) * 100), 2);
      }
    }

    return $data;
  }

  /**
   * @param TestSession[] $sessions
   * @return array
   */
  private function getSpeedData(array $sessions, Experiment $experiment) {
    $data = [];

    foreach($sessions as $session) {
      if ($session['number'] !== 1 && $session['number'] !== $experiment->getTotalSessions()) {
        $data[] = (float)($session['medianInterval'] / 1000 );
      }
    }

    return $data;
  }

  private function getScoreData(array $sessions, Experiment $experiment) {
    $data = [];

    foreach($sessions as $session) {
      if ($session['number'] !== 1 && $session['number'] !== $experiment->getTotalSessions()) {
        $data[] = $session['totalScore'];
      }
    }

    return $data;
  }

}
