<?php

namespace WebsiteBundle\Twig;

use Bazookas\AdminBundle\AdminElements\Containers\MenuContainer;
use Bazookas\AdminBundle\AdminElements\Elements\Actions\MenuActions\MenuActionElement;
use Bazookas\AdminBundle\Twig\Base\BaseAdminMenuExtension;
use Symfony\Component\HttpFoundation\Request;
use WebsiteBundle\Security\Roles;

class AdminMenuExtension extends BaseAdminMenuExtension
{

  protected function createMenu(Request $request) {
    $this->addMenuHeader();

    //Add dashboard menu item
    $this->elements[] = new MenuActionElement([], [
      'label' => 'admin.menu.dashboard.label',
      'route' => 'bazookas_admin_dashboard',
      'iconClass' => 'fa fa-tachometer fa-3x',
      'roles' => [ Roles::ROLE_CMS_USER ],
    ]);

    //Add participants menu item
    $this->elements[] = new MenuActionElement([], [
      'label' => 'admin.menu.participants.label',
      'route' => 'admin_participants',
      'iconClass' => 'fa fa-users fa-3x',
      'roles' => [ Roles::ROLE_CMS_USER ],
    ]);

    //Add experiments menu item
    $this->elements[] = new MenuActionElement([], [
      'label' => 'admin.menu.experiments.label',
      'route' => 'admin_experiments',
      'iconClass' => 'fa fa-flask fa-3x',
      'roles' => [ Roles::ROLE_CMS_USER ],
    ]);

    //Add test menu item
    $this->elements[] = new MenuActionElement([], [
      'label' => 'admin.menu.test.label',
      'route' => 'admin_test',
      'iconClass' => 'fa fa-file-text-o fa-3x',
      'roles' => [ Roles::ROLE_ADMIN_RESEARCHER ],
    ]);

    //Add quote menu item
    $this->elements[] = new MenuActionElement([], [
      'label' => 'admin.menu.quote.label',
      'route' => 'admin_quote',
      'iconClass' => 'fa fa-quote-right  fa-3x',
      'roles' => [ Roles::ROLE_CMS_USER ],
    ]);

    $this->addAdminBundleMenuItems();
    $this->addMediaBundleMenuItems();
    $this->addCronBundleMenuItems();
  }

  /**
   * @inheritdoc
   */
  protected function addAdminBundleMenuItems() {
    //Add the user management menu items
    $this->elements[] = new MenuContainer([
      new MenuActionElement([], [
        'label' => 'admin.entities.researcher.namePlural',
        'route' => 'admin_cms_user',
        'iconClass' => 'ti ti-user',
        'roles' => [Roles::ROLE_ADD_USER],
      ]),
      new MenuActionElement([], [
        'label' => 'admin.entities.cMSUserGroup.namePlural',
        'route' => 'admin_cms_user_group',
        'iconClass' => 'ti-id-badge',
        'roles' => [Roles::ROLE_ADD_GROUP],
      ])
      ],
      [
        'label' => 'admin.menu.researcher.label',
        'roles' => [Roles::ROLE_ADD_USER, Roles::ROLE_ADD_GROUP],
        'iconClass' => 'ti ti-user'
      ]);

    //Add the copy module menu items
    $this->elements[] = new MenuActionElement([], [
      'label' => 'admin.menu.copy.label',
      'route' => 'admin_copy',
      'iconClass' => 'fa fa-language fa-3x',
      'roles' => [ Roles::ROLE_COPY_MANAGER ],
    ]);
  }

}
