<?php

namespace Bazookas\AdminBundle\Controller\AdminController;

use Bazookas\AdminBundle\Controller\Base\BaseAdminListController;
use Bazookas\AdminBundle\Entity\CMSUser;
use Bazookas\AdminBundle\Form\CMSUserAdminType;
use Bazookas\AdminBundle\PageBuilder\Interfaces\ListPageBuilderInterface;
use Bazookas\AdminBundle\PageBuilder\ListPageBuilder;
use Bazookas\CommonBundle\Security\Roles;
use Symfony\Component\HttpFoundation\Request;

class BaseCMSUserAdminController extends BaseAdminListController
{

  protected function getEntityClass()
  {
    return CMSUser::class;
  }

  protected function getFormClass($action) {
    switch($action) {
      case self::ACTION_ADD:
        return CMSUserAdminType::class;
      case self::ACTION_EDIT:
        return CMSUserAdminType::class;
      default:
        return null;
    }
  }

  protected function modifyListBuilder(Request $request, ListPageBuilderInterface $builder)
  {
    /**
     * @var ListPageBuilder $builder
     */
    $builder = parent::modifyListBuilder($request, $builder);

    $builder
      ->addBooleanField('enabled')
      ->addField('username')
      ->addField('email')
      ->addField('groups', [
        'template' => 'BazookasAdminBundle:List/Fields/CMSUser:_groups.html.twig'
      ])

      ->addTextFilterField('username')
      ->addTextFilterField('email')
      ->addTextFilterField('groups')
      ->addBooleanFilterField('enabled')
      ->setMaintainFilterState(true)
    ;

    return $builder;
  }

  protected function hasAccess($action) {
    switch($action) {
      case self::ACTION_LIST:
      case self::ACTION_ADD:
      case self::ACTION_EDIT:
      case self::ACTION_REMOVE:
        return $this->isGranted(Roles::ROLE_ADD_USER);
      default:
        return false;
    }
  }

}
