<?php

namespace Bazookas\AdminBundle\Controller\Base;

use Bazookas\AdminBundle\Dashboard\Base\BaseDashboardConfigurator;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Symfony\Component\HttpFoundation\Request;

/**
 * @deprecated
 * Class DashboardBaseController
 * @package Bazookas\AdminBundle\Controller\Base
 */
abstract class DashboardBaseController extends Controller {

  protected $config;

  public function __construct() {
    $this->config = new BaseDashboardConfigurator();
  }

  public function indexAction(Request $request) {
    if (!$this->hasAccess()) {
      return $this->accessDeniedRedirect($request);
    }

    $templateData = $this->getTemplateData($request);

    $this->buildPage($request);

    return $this->render(
      $this->config->getLayoutTemplate(), array(
      'config' => $this->config,
      'templateData' => $templateData,
      'styles' => $this->config->getStyles(),
      'scripts' => $this->config->getScripts(),
    ));
  }

  protected function getTemplateData(Request $request) {
    return array('titleTranslationKey' => $this->config->getTitle());
  }

  protected function accessDeniedRedirect(Request $request) {
    $translator = $this->get('translator');
    $this->addFlash(
      'error',
      $translator->trans('admin.list.messages.accessDenied', [], 'admin')
    );

    //send the user back where they belong!
    $route = $request->get('_route');
    $routeParams = $request->get('_route_params');

    if (empty($route)) {
      throw new AccessDeniedException();
    }

    return $this->redirectToRoute($route, $routeParams);
  }

  protected abstract function hasAccess();

  protected abstract function buildPage(Request $request);
}
