<?php

namespace Bazookas\AdminBundle\Entity\Base;

use Bazookas\AdminBundle\Entity\Interfaces\Notification\NotificationInterface;
use Bazookas\CommonBundle\Entity\Base\BaseCMSUser;
use Bazookas\CommonBundle\Entity\Base\BaseEntity;
use Bazookas\CommonBundle\Security\Roles;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\MappedSuperclass
 */
abstract class BaseNotification extends BaseEntity implements NotificationInterface {

  /**
   * @ORM\Column(type="string")
   * @Assert\NotBlank()
   * @var string
   */
  protected $message;

  /**
   * @ORM\Column(type="string", nullable=true)
   * @var string
   */
  protected $link;

  /**
   * Unmapped field to be used as the discriminator by doctrine
   * @var string
   */
  protected $type;

  /**
   * @var BaseCMSUser
   */
  protected $user;

  /**
   * @return string
   */
  public function getMessage()
  {
    return $this->message;
  }

  /**
   * @param string $message
   * @return $this
   */
  public function setMessage(string $message)
  {
    $this->message = $message;

    return $this;
  }

  /**
   * @return string
   */
  public function getLink()
  {
    return $this->link;
  }

  /**
   * An absolute URL.
   * @param string $link
   * @return $this
   */
  public function setLink(string $link)
  {
    $this->link = $link;

    return $this;
  }

  /**
   * @return string
   */
  public function getType()
  {
    return $this->type;
  }

  /**
   * @param string $type
   * @return $this
   */
  public function setType(string $type)
  {
    $this->type = $type;

    return $this;
  }

  /**
   * @return BaseCMSUser
   */
  public function getUser()
  {
    return $this->user;
  }

  /**
   * @param BaseCMSUser $user
   * @return $this
   */
  public function setUser(BaseCMSUser $user)
  {
    $this->user = $user;

    return $this;
  }

  /**
   * @override
   * @return string
   */
  public function getRoleRequiredForView() {
    return Roles::ROLE_CMS_USER;
  }
}
