<?php

namespace Bazookas\AdminBundle\Entity;

use Bazookas\CommonBundle\Security\Roles;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use FOS\UserBundle\Model\GroupInterface;
use Doctrine\Common\Collections\ArrayCollection;
use Bazookas\CommonBundle\Entity\Base\BaseEntity;

  /**
  * @ORM\Table(name="cms_user_group")
  * @ORM\Entity(repositoryClass="Bazookas\AdminBundle\Repository\CMSUserGroupRepository")
  */
class CMSUserGroup extends BaseEntity implements GroupInterface
{
  /**
   * @ORM\Column(type="string")
   * @var string
   **/
  protected $name;

  /**
   * @ORM\Column(type="array")
   * @var array
   **/
  protected $roles;

  /**
   * Use this field in case you need to uniquely identify a group when working with default groups
   *
   * @ORM\Column(type="string", nullable=true)
   * @var string
   */
  protected $systemId;

  public function __construct() {
    parent::__construct();
  }

  /**
   * @return mixed
   */
  public function getName() {
    return $this->name;
  }

  /**
   * @param string $name
   * @return $this
   */
  public function setName($name) {
    $this->name = $name;
    return $this;
  }

  /**
   * @return array
   */
  public function getRoles()
  {
    return $this->roles;
  }

  /**
   * @param array $roles
   * @return $this
   */
  public function setRoles(array $roles) {
    $this->roles = $roles;
    return $this;
  }

  /**
   * @param string $role
   * @return $this
   */
  public function addRole($role)
  {
    if (empty($this->roles)) {
      $this->roles = array();
    }

    $roles = new ArrayCollection($this->roles);

    if (!$roles->contains($role)) {
      $roles->add($role);
    }

    $this->roles = $roles->toArray();

    return $this;
  }

  /**
   * @param string $role
   * @return $this
   */
  public function removeRole($role)
  {
    $roles = new ArrayCollection($this->roles);

    if ($roles->contains($role)) {
      $roles->removeElement($role);
    }

    $this->roles = $roles->toArray();

    return $this;
  }

  /**
   * @param string $checkRole
   * @return bool
   */
  public function hasRole($checkRole) {
    foreach($this->roles as $role) {
      if ($role === $checkRole) {
        return true;
      }
    }

    return false;
  }

  /**
   * @return string
   */
  public function getSystemId()
  {
    return $this->systemId;
  }

  /**
   * @param string $systemId
   * @return $this
   */
  public function setSystemId(string $systemId)
  {
    $this->systemId = $systemId;

    return $this;
  }

  /**
  * overwrite of BaseEntity method
  */
  public function getRoleRequiredForView() {
    return Roles::ROLE_ADD_GROUP;
  }
}
