<?php

namespace Bazookas\AdminBundle\Form;

use Bazookas\AdminBundle\Filter\ListFilter;
use Bazookas\AdminBundle\Filter\ListFilterField;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Translation\TranslatorInterface;

class ListFilterForm extends AbstractType
{
  private $translator;

  public function __construct(TranslatorInterface $translator)
  {
    $this->translator = $translator;
  }

  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    // NOTE: for some dumb reason, $builder->getData() returns null here. So we wait for the data to get filled in with an event listener
    $builder
      ->addEventListener(FormEvents::POST_SET_DATA, [$this, 'configureFilterFields'])
    ;
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults(array(
      'data_class' => ListFilter::class,
      'translation_domain' => 'admin'
    ));
  }

  public function configureFilterFields(FormEvent $event) {
    if (null != $event->getData()) {
      $form = $event->getForm();
      $data = $event->getData();

      $form
        ->remove('field')
        ->remove('operator')
        ->remove('value')
        ->remove('type')
      ;

      $form
        ->add('field', ChoiceType::class, array(
          'label' => false,
          'choices' => $data->getFilterFields(),
          'choice_label' => [$this, 'getFieldChoiceLabel'],
          'choice_value' => [$this, 'getFieldChoiceValue'],
          'choice_attr' => [$this, 'getFieldChoiceAttr'],
          'attr' => array(
            'class' => 'js-filter-field select2'
          )
        ))
        ->add('operator', ChoiceType::class, array(
          'label' => false,
          'attr' => array(
            'class' => 'js-filter-operator select2',
            'data-initially-selected' => $data->getOperator()
          ),
          'choices' => ListFilterField::getDefaultOperators()
        ))
        ->add('value', TextType::class, array(
          'label' => false,
          'attr' => array(
            'class' => 'js-filter-value'
          ),
          'required' => false,
          'empty_data' => false
        ))
        ->add('type', HiddenType::class, array(
          'attr' => array(
            'class' => 'js-filter-type'
          ),
        ))
      ;
    }
  }

  public function getFieldChoiceLabel($field) {
    if ($field instanceof ListFilterField) {
      return $field->getLabel();
    }
    return $field;
  }
  public function getFieldChoiceValue($field) {
    if ($field instanceof ListFilterField) {
      return $field->getField();
    }
    return $field;
  }

  public function getFieldChoiceAttr($field) {
    $attributes = [];
    if ($field instanceof ListFilterField) {
      //prepare the operators
      $operators = [];
      foreach($field->getOperators() as $key => $operator) {
        // translate the keys for rendering in frontend
        $operators[$this->translator->trans($key, [], 'admin')] = $operator;
      }

      $attributes['data-operators'] = json_encode($operators);
      $attributes['data-type'] = $field->getType();

      // add the allowed values (for choice types)
      $untranslatedChoices = $field->getChoices();
      $choices = [];
      //first we need to translate the key-value pairs
      foreach($untranslatedChoices as $label => $value) {
        $label = $this->translator->trans($label, [], 'admin');
        $value = $this->translator->trans($value, [], 'admin');

        $choices[$label] = $value;
      }

      if (!empty($choices)) {
        $attributes['data-choices'] = json_encode($choices);
      }
    }

    return $attributes;
  }
}
