<?php

namespace Bazookas\AdminBundle\Repository\Base;

use Bazookas\AdminBundle\Entity\Interfaces\Notification\NotificationInterface;
use Bazookas\AdminBundle\Repository\Interfaces\Notification\CMSNotificationRepositoryInterface;
use Bazookas\CommonBundle\Entity\Base\BaseCMSUser;
use Doctrine\ORM\Query;
use Doctrine\ORM\QueryBuilder;

class BaseCMSNotificationRepository extends BaseNotificationRepository implements CMSNotificationRepositoryInterface
{

  /**
   * @inheritdoc
   */
  public function beforeListQueryExecution(QueryBuilder $qb, array $params) {
    $rootAlias = $qb->getRootAliases()[0];
    $user = $params['user'];

    $qb
      ->andWhere($qb->expr()->eq($rootAlias.'.user', ':user'))
      ->setParameter('user', $user)
      ->addOrderBy($rootAlias.'.created', 'DESC')
    ;

    return $qb;
  }

  /**
   * @param BaseCMSUser $user
   * @param int $hydrationMode
   * @return NotificationInterface[]
   */
  public function getCMSNotifications(BaseCMSUser $user, $hydrationMode = Query::HYDRATE_ARRAY) {
    $qb = $this->_em->createQueryBuilder();
    $qb
      ->select('notifications')
      ->from($this->_entityName, 'notifications')
      ->andWhere($qb->expr()->eq('notifications.user', ':user'))
      ->setParameter('user', $user)
      ->addOrderBy('notifications.created', 'DESC')
      ->setMaxResults(10)
    ;

    return $qb->getQuery()->getResult($hydrationMode);
  }

  /**
   * @param BaseCMSUser $user
   * @param array $ids
   */
  public function markNotificationsAsViewed(BaseCMSUser $user, array $ids) {
    $qb = $this->_em->createQueryBuilder();
    $qb
      ->update($this->_entityName, 'notifications')
      ->set('notifications.new', ':viewed')
      ->andWhere($qb->expr()->in('notifications.id', ':ids'))
      ->andWhere($qb->expr()->eq('notifications.user', ':user'))
      ->setParameters([
        'viewed' => false,
        'ids'=> $ids,
        'user' => $user
      ])
    ;

    $qb->getQuery()->execute();
  }

  /**
   * @param BaseCMSUser $user
   * @param array $ids
   */
  public function removeNotifications(BaseCMSUser $user, array $ids) {
    $qb = $this->_em->createQueryBuilder();
    $qb
      ->delete($this->_entityName, 'notifications')
      ->andWhere($qb->expr()->in('notifications.id', ':ids'))
      ->andWhere($qb->expr()->eq('notifications.user', ':user'))
      ->setParameters([
        'ids'=> $ids,
        'user' => $user
      ])
    ;
    $qb->getQuery()->execute();
  }

}
