<?php

namespace Bazookas\AdminBundle\Tests\Controller\AdminController;

use Bazookas\AdminBundle\Controller\Base\BaseAdminListController;
use Bazookas\AdminBundle\Entity\CMSUserGroup;
use Bazookas\AdminBundle\Tests\TestCase\AdminTestCase;

class BaseUserGroupAdminControllerTest extends AdminTestCase
{

  /**
   * Test that pages load successfully when logged in as admin
   * @return array
   */
  public function getUrls() {
    $urls = [
      $this->getUrl('admin_cms_user_group', [
        '_locale' => $this->defaultLocale,
        '_action' => BaseAdminListController::ACTION_LIST
      ]),
      $this->getUrl('cms_user_group_admin', [
        '_locale' => $this->defaultLocale,
        '_action' => BaseAdminListController::ACTION_LIST
      ])
    ];

    $parentUrls = parent::getUrls();
    return array_merge($urls, $parentUrls);
  }

  public function testAddGroup() {
    list($client, $crawler) = $this->executeRequest('admin_cms_user_group', [
      '_locale' => $this->defaultLocale,
      '_action' => BaseAdminListController::ACTION_ADD
    ]);

    $form = $crawler->selectButton('submit')->form();
    $form['cms_user_group_admin[name]'] = 'test';
    $form['cms_user_group_admin[roles][0]']->tick();
    $client->submit($form);

    //Assert that the form submitted successfully
    $this->assertTrue($client->getResponse()->isRedirect());

    //Assert that the new item was added
    $entityManager = $client->getContainer()->get('doctrine.orm.entity_manager');
    $entityRepo = $entityManager->getRepository(CMSUserGroup::class);
    $entity = $entityRepo->findOneBy(['name' => 'test']);

    $this->assertTrue(!empty($entity));
  }

  public function testEditGroup() {
    list($client, $crawler) = $this->executeRequest('admin_cms_user_group', [
      '_locale' => $this->defaultLocale,
      '_action' => BaseAdminListController::ACTION_EDIT,
      '_id' => 1
    ]);

    $form = $crawler->selectButton('submit')->form();
    $form['cms_user_group_admin[name]'] = 'test2';
    $client->submit($form);

    //Assert that the form submitted successfully
    $this->assertTrue($client->getResponse()->isRedirect());

    //Assert that the item was edited
    $entityManager = $client->getContainer()->get('doctrine.orm.entity_manager');
    $entityRepo = $entityManager->getRepository(CMSUserGroup::class);
    $entity = $entityRepo->findOneBy(['name' => 'test']);

    $this->assertTrue(empty($entity));

    $entity = $entityRepo->findOneBy(['name' => 'test2']);
    $this->assertTrue(!empty($entity));
  }

  public function testRoleSuperAdminUser() {
    $adminUser = $this->referenceRepository->getReference('user-super-admin');

    $this->testPageIsSuccessful($adminUser, [
      $this->getUrl('admin_cms_user_group', [
        '_locale' => $this->defaultLocale,
        '_action' => BaseAdminListController::ACTION_LIST
      ])
    ]);
  }

  public function testRequireRoleAdminUser() {
    //Test basic ROLE_CMS_USER
    $cmsUser = $this->referenceRepository->getReference('user-basic1');
    $this->testPageFails($cmsUser, [
      $this->getUrl('admin_cms_user_group', [
        '_locale' => $this->defaultLocale,
        '_action' => BaseAdminListController::ACTION_LIST
      ]),
      $this->getUrl('admin_cms_user_group', [
        '_locale' => $this->defaultLocale,
        '_action' => BaseAdminListController::ACTION_ADD
      ]),
      $this->getUrl('cms_user_group_admin', [
        '_locale' => $this->defaultLocale,
        '_action' => BaseAdminListController::ACTION_EDIT
      ])
    ]);

    //Test role admin has no access
    $cmsUser = $this->referenceRepository->getReference('user-admin1');
    $this->testPageFails($cmsUser, [
      $this->getUrl('admin_cms_user_group', [
        '_locale' => $this->defaultLocale,
        '_action' => BaseAdminListController::ACTION_LIST
      ]),
      $this->getUrl('admin_cms_user_group', [
        '_locale' => $this->defaultLocale,
        '_action' => BaseAdminListController::ACTION_ADD
      ]),
      $this->getUrl('cms_user_group_admin', [
        '_locale' => $this->defaultLocale,
        '_action' => BaseAdminListController::ACTION_EDIT
      ])
    ]);
  }

}
