<?php

namespace Bazookas\AdminBundle\Tests\TestCase;

use Bazookas\AdminBundle\Entity\CMSUser;
use Bazookas\AdminBundle\Tests\DataFixtures\AdminTestDataFixtures;
use Doctrine\Common\DataFixtures\ReferenceRepository;
use Liip\FunctionalTestBundle\Test\WebTestCase;

abstract class AdminTestCase extends WebTestCase
{
  /**
   * @var ReferenceRepository
   */
  protected $referenceRepository;

  protected $defaultLocale = 'en';

  public function setUp() {
    $this->referenceRepository = $this->loadFixtures($this->getFixtures())->getReferenceRepository();
  }

  /**
   * Overwrite this function as needed
   * @return array a list of fully qualified classnames of doctrine fixtures
   */
  protected function getFixtures() {
    return [
      AdminTestDataFixtures::class
    ];
  }

  /**
   * Create a client logged in with the $adminUser.
   * @param CMSUser $adminUser the user with which to run the tests (default super admin)
   * @return \Symfony\Bundle\FrameworkBundle\Client
   */
  protected function createLoggedInClient(CMSUser $adminUser = null) {
    if (empty($adminUser)) {
      $adminUser = $this->referenceRepository->getReference('user-super-admin');
    }

    //login as the $adminUser in the admin firewall
    $this->loginAs($adminUser, 'admin');
    $client = $this->makeClient();
    return $client;
  }

  /**
   * @param CMSUser $adminUser the user with which to run the tests (default super admin)
   */
  public function testPageIsSuccessful(CMSUser $adminUser = null, $urls = []) {
    if (empty($urls)) {
      $urls = $this->getUrls();
    }

    $client = $this->createLoggedInClient($adminUser);

    foreach($urls as $url) {
      $client->request('GET', $url);
      $this->isSuccessful($client->getResponse());
    }
  }

  /**
   * Overwrite as needed
   * @return array a list of routes which should load (no parameters)
   */
  public function getUrls()
  {
    //Use relative urls
    return [
      $this->getUrl('bazookas_admin_dashboard', ['_locale' => $this->defaultLocale])
    ];
  }

  /**
   * Overwrite as needed
   *
   * Test that pages fail
   * @param CMSUser $adminUser the user with which to run the tests (default not logged in)
   */
  public function testPageFails(CMSUser $adminUser = null, $urls = []) {
    if (empty($urls)) {
      $urls = $this->getUrls();
    }

    if (!empty($adminUser)) {
      $this->loginAs($adminUser, 'admin');
    }
    $client = $this->makeClient();

    foreach($urls as $url) {
      $client->request('GET', $url);
      $this->isSuccessful($client->getResponse(), false);
    }
  }

  public function testAdminHomepageRedirect() {
    $client = $this->makeClient();
    $path = $this->getUrl('bazookas_admin_homepage', ['_locale' => $this->defaultLocale]);

    $client->request('GET', $path);
    $this->assertStatusCode(302, $client);
    $this->assertTrue($client->getResponse()->isRedirect());
  }

  /**
   * @param $route
   * @param array $routeParams
   * @param string $method
   * @return array[]
   */
  protected function executeRequest($route, $routeParams = [], $method = 'GET') {
    $client = $this->createLoggedInClient();
    $url = $this->getUrl($route, $routeParams);

    $crawler = $client->request($method, $url);

    return [$client, $crawler];
  }

}
