const TASKS = require('gulpinator/utilities/taskNames');
const FS = require('fs');
const BOWER_ROOT = './vendor/bower/';

const ADMIN_ROOT = './vendor/bazookas/admin-bundle/Resources/public/';
const ADMIN_SUBMODULE_ROOT = './src/Bazookas/AdminBundle/Resources/public/';
const IS_ADMIN_SUBMODULE = FS.existsSync(ADMIN_SUBMODULE_ROOT);

let defaultConfig = {
  files: [
    // compile admin scss
    {
      task: TASKS.styles,
      target: (IS_ADMIN_SUBMODULE ? ADMIN_SUBMODULE_ROOT : ADMIN_ROOT) + 'css/**/*.scss',
      options: {
        dest: 'styles/admin',
        sourcemaps: true,
        watch: true
      }
    },

    // extra admin styles
    // TODO make it possible to dump into one file
    {
      task: TASKS.styles,
      target: [
        BOWER_ROOT + 'PACE/themes/blue/pace-theme-minimal.css',
        BOWER_ROOT + 'bootstrap/dist/css/bootstrap.min.css',
        BOWER_ROOT + 'select2/dist/css/select2.min.css',
        BOWER_ROOT + 'bootstrap-daterangepicker/daterangepicker.css',
        (IS_ADMIN_SUBMODULE ? ADMIN_SUBMODULE_ROOT : ADMIN_ROOT) + '/plugins/bootstrap/timepicker/bootstrap-timepicker.min.css'
      ],
      options: {
        name: 'admin-extra-styles',
        dest: 'styles/admin'
      }
    },

    // Moving images
    {
      task: TASKS.move,
      target: [
        (IS_ADMIN_SUBMODULE ? ADMIN_SUBMODULE_ROOT : ADMIN_ROOT) + 'images/**'
      ],
      options: {
        dest: 'img'
      }
    },

    // move additional files
    // (fonts, etc.)
    {
      task: TASKS.move,
      target: [
        (IS_ADMIN_SUBMODULE ? ADMIN_SUBMODULE_ROOT : ADMIN_ROOT) + 'other/**'
      ]
    },

    // Bundle bower javascript libraries
    {
      task: TASKS.jsBundle,
      target: [
        BOWER_ROOT + 'jquery/dist/jquery.min.js',
        BOWER_ROOT + 'bootstrap/dist/js/bootstrap.min.js',
        BOWER_ROOT + 'bootstrap-datepicker/dist/js/bootstrap-datepicker.min.js',
        BOWER_ROOT + 'bootstrap-timepicker/js/bootstrap-timepicker.js',
        BOWER_ROOT + 'bootstrap-datepicker/dist/locales/bootstrap-datepicker.en-GB.min.js',
        BOWER_ROOT + 'bootstrap-datepicker/dist/locales/bootstrap-datepicker.nl-BE.min.js',
        BOWER_ROOT + 'select2/dist/js/select2.min.js',
        BOWER_ROOT + 'bootbox.js/bootbox.js',
        BOWER_ROOT + 'Sortable/sortable.min.js',
        BOWER_ROOT + 'urijs/src/URI.min.js',
        BOWER_ROOT + 'PACE/pace.min.js',
        BOWER_ROOT + 'chart.js/dist/Chart.min.js',
        BOWER_ROOT + 'moment/min/moment.min.js',
        (IS_ADMIN_SUBMODULE ? ADMIN_SUBMODULE_ROOT : ADMIN_ROOT) + '/plugins/nifty/nifty.min.js'
      ],
      options: {
        name: 'admin-libs',
        dest: 'scripts',
        //Don't minify them as it causes issues.. for... reasons
        minify: false
      }
    },
    {
      task: TASKS.jsBundle,
      target: [
        BOWER_ROOT + 'bootstrap-daterangepicker/daterangepicker.js'
      ],
      options: {
        name: 'date-range-picker',
        dest: 'scripts',
        minify: true
      }
    },

    // custom admin scripts
    {
      task: TASKS.jsBundle,
      target: [
        (IS_ADMIN_SUBMODULE ? ADMIN_SUBMODULE_ROOT : ADMIN_ROOT) + '/js/**/*.js'
      ],
      options: {
        name: 'admin-scripts',
        dest: 'scripts',
        minify: true,
        watch: true,
        sourceMaps: true,
      }
    }

  ],
  options: {
    dest: 'web'
  }
};

module.exports = {
  default: defaultConfig
};
