<?php

namespace Bazookas\APIFrameworkBundle\Controller;

use Bazookas\APIFrameworkBundle\Services\Validation\ValidationService;
use Symfony\Component\HttpFoundation\Request;
use Bazookas\APIFrameworkBundle\Controller\Base\BaseController ;
use Nelmio\ApiDocBundle\Annotation\ApiDoc;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\Intl\Exception\NotImplementedException;
use Symfony\Component\Validator\Constraints\DateTime;

class AppCopyController extends BaseController
{
  public function getDataService($dataServiceFunction)
  {
    return $this->get('APIFrameworkBundle.Data.AppCopyDataService');
  }


  /**
   * Return a list of key-value pairs used as translations in the app.
   *
   * ### Example response
   *
   *     {
   *      "data": [
   *        {
   *          "key": "ACCOUNT_LABELMYDISHES",
   *          "value": "gerechten"
   *        }
   *      ],
   *      "lastModified": "1484050577"
   *     }
   *
   *
   * @ApiDoc(
   *   section="Settings",
   *  resource=true,
   *  description="",
   *  requirements={},
   *  headers={
   *   {
   *     "name"="Authorization",
   *     "description"="Used to send the access token to authenticate the app to the API. The header is formatted as follows: 'Authorization':'Bearer < token >'.",
   *     "required" = true
   *   },
   *  },
   *  parameters={
   *   {
   *    "name"="newer",
   *    "dataType"="timestamp",
   *    "required"=false,
   *    "description"="get translations created or modified after given timestamp"
   *   },
   *  },
   *  statusCodes={
   *    200="Returned when successful",
   *    400="Missing or invalid parameter",
   *    404="Not found (when language does not exist)",
   *  },
   *  tags={
   *    "requires-client_credentials" = "#ffa500",
   *  }
   *)
   */
  public function getAction(Request $request, $_locale) {
    $retrieveParams = array(
      'newer' => [
        'constraint' => [new DateTime(['format' => 'U'])],
        'convertToType' => ValidationService::TYPE_DATETIME,
        'conversionFormat' => 'U',
      ]
    );
    $extraParams = new ParameterBag([
      'locale' => $_locale,
    ]);

    return $this->handleRequest($request, $retrieveParams, 'getAll', Response::HTTP_OK, $extraParams);
  }
}
