<?php

namespace Bazookas\APIFrameworkBundle\Controller;

use Bazookas\APIFrameworkBundle\Controller\Base\BaseController;
use Bazookas\APIFrameworkBundle\Exception\InvalidArgumentException;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\HttpFoundation\Request;
use Nelmio\ApiDocBundle\Annotation\ApiDoc;
use Symfony\Component\HttpFoundation\Response;

class FileUploadController extends BaseController
{

  /**
   * Post a single file to this endpoint.
   *
   * ### Post body ###
   *
   * raw file data
   *
   * ### Successfull response ###
   *
   *     {
   *       "data": "http://domain.com/path/to/upload.file"
   *     }
   *
   * @ApiDoc(
   *  resource=true,
   *  description="upload raw file",
   *  requirements={},
   *  headers={
   *   {
   *     "name"="Authorization",
   *     "description"="Used to send the access token to authenticate the app to the API. The header is formatted as follows: 'Authorization':'Bearer < token >'.",
   *     "required" = true
   *   },
   *   {
   *     "name"="Content-Type",
   *     "description"="The type (mime-type) of file you are sending, like 'image/png'",
   *     "required"=true
   *   },
   *   {
   *     "name"="Content-Length",
   *     "description"="The length of the content (in bytes)",
   *     "required"=true
   *   },
   *   {
   *     "name"="Content-Range",
   *     "description"="NOT IMPLEMENTED, see for example: https://developers.google.com/youtube/v3/guides/using_resumable_upload_protocol#Resume_Upload",
   *     "required"=false
   *   },
   *   {
   *     "name"="FileName",
   *     "description"="Optional file name, if a preferred file name is already set in the database this will be ignored. (Recommended?)",
   *     "required"=false
   *   }
   *  },
   *  statusCodes={
   *    200="Returned when successful"
   *  },
   *  tags={
   *  }
   * )
   **/
  public function uploadAction(Request $request, $_hash, $_index) {
    $extraParams = new ParameterBag([
      'contentType' => $request->headers->get('Content-Type'),
      'contentLength' => intval($request->headers->get('Content-Length')),
      'fileName' => $request->headers->get('FileName'),

      'file' => $request->getContent(),
      'hash' => $_hash,
      'index' => $_index,
    ]);
    return $this->handleRequest($request, [], 'uploadFullFile', Response::HTTP_OK, $extraParams);
  }

}
