<?php

namespace Bazookas\APIFrameworkBundle\Form;

use Bazookas\CommonBundle\Form\Type\LocaleType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

class AppCopyDetailAdminType extends AbstractType
{
  private $requiredLocales;
  private $entityClass;

  public function __construct($requiredLocales, $entityClass)
  {
    $this->requiredLocales = $this->requiredLocalesAsChoiceArray($requiredLocales);
    $this->entityClass = $entityClass;
  }

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('locale', LocaleType::class, [
        'label' => false,
        'choices' => $this->requiredLocales,
      ])
      ->add('value', TextareaType::class, [
        'label' => 'admin.entities.appCopy.fields.value'
      ])
    ;
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults([
      'data_class' => $this->entityClass
    ]);
  }


  private function requiredLocalesAsChoiceArray($requiredLocales) {
    $requiredLocales = explode('|', $requiredLocales);

    $locales = [];
    foreach($requiredLocales as $locale) {
      $locales[$locale] = $locale;
    }

    return $locales;
  }
}
