## Reference
### Injecting the service

A service named `BazookasAPIFrameworkBundle.upload.fileUploadService` is provided for generating upload URLs.   
It can be injected into other services or retrieved directly from the container when container aware.
```
  apibundle.dataService.AppUserDataService:
    class: APIBundle\Services\Data\AppUserDataService
    arguments:
      - "@BazookasAPIFrameworkBundle.upload.fileUploadService"
```

### Options explained

```php
$uploadTargets = $this->uploadService->createFileUploads([
  [
    'targetDirectory' => 'images/profile',
    'overwrite' => true,
    'filename' => 'user_'.$user->getId(),
    'mimeTypes' => '#^image/.+$#',
    'callback' => new APIFileUploadCallback(
      'BazookasMediaBundle.dataService.APICallBackService',
      'uploadComplete',
      ['id' => $image->getId()]
    ),
  ],
]);
```

- **targetDirectory**
  - where the image should be uploaded
  - starting from the upload folder set in config
- **overwrite**
  - default: false
  - allow file overwrites
  - only use when generating own unique names
- **filename**
  - preferred file name for upload
  - try to create a sensible name for easily finding files
- **mymeTypes**
  - default: /.+/
  - regex
  - use this to allow only these mime types
  - default no restriction
  - example uses # delimiters to prevent having to escape slashes
  - to allow multiple mime types use the regex | special character
- **callback**
  - add a callback to the upload
  - when a single file is completed
  - `@BazookasMediaBundle.dataService.APICallBackService` will set URL and file path in a media entity
  - a global callback (when ALL FILES are completed) can be added a second parameter to the `createFileUploads` method
