<?php

namespace Bazookas\APIFrameworkBundle\Services\Data\Base;

use Bazookas\CommonBundle\Utility\Utility;
use Bazookas\CommonBundle\Repository\Interfaces\RepositoryInterface;
use Bazookas\APIFrameworkBundle\Services\Data\Interfaces\DataServiceInterface;
use Doctrine\Common\Persistence\ObjectRepository;

abstract class BaseDataService implements DataServiceInterface {
  /**
   * @var $repository ObjectRepository
   */
  protected $repository;
  protected $formatter;

  public function __construct(RepositoryInterface $repository = null, $formatter = null) {
    $this->repository = $repository;
    $this->formatter = $formatter;
  }

  public function getRepository() {
    return $this->repository;
  }

  public function setRepository(RepositoryInterface $repository) {
    $this->repository = $repository;
    return $this;
  }

  public function getFormatter() {
    return $this->formatter;
  }

  public function setFormatter($formatter) {
    $this->formatter = $formatter;
    return $this;
  }

  public function getLastModified($lastModified, &$data, $unset = false) {

    $dataLastModified = $lastModified;
    if (isset($data['modified'])) {
      $dataLastModified = $data['modified'];
      if ($unset) {
        unset($data['modified']);
      }
    }

    //filter so we're only left with subelements
    foreach($data as &$element) {
      if (is_array($element)) {
        $dataLastModified = $this->getLastModified($dataLastModified, $element, $unset);
      }
    }

    return $dataLastModified > $lastModified ? $dataLastModified : $lastModified ;
  }

}
