<?php

namespace Bazookas\CommonBundle\Command\Setup;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Filesystem\LockHandler;
use Sensio\Bundle\DistributionBundle\Composer\ScriptHandler;
use Symfony\Component\Process\Process;
use Symfony\Component\Process\Exception\ProcessFailedException;

/**
 * @deprecated to be removed in v2
 *
 * Class InitialSetupCommand
 * @package Bazookas\CommonBundle\Command\Setup
 */
class InitialSetupCommand extends ContainerAwareCommand
{
  private $output;
  private $command = 'project:install:dependencies';

  protected function configure()
  {
    $this
      ->setName($this->command)
      ->setDescription("Runs the intial setup of the project")
    ;
  }

  protected function execute(InputInterface $input, OutputInterface $output)
  {
    $this->output = $output;
    $commandLock = new LockHandler($this->command);

    if (!$commandLock->lock()) {
      $output->writeln('<error>The command is already running in another process.</error>');
    } else {
      $output->writeln("<comment>".date('Y-m-d H:i:s')." - Starting initial setup...</comment>");

      //install npm packages
      $this->installNpmPackages($output);

      //run bower install
      $this->gulpBowerInstall($output);

      //run gulp build
      $this->gulpBuild($output);

      $output->writeln("<info>".date('Y-m-d H:i:s')." - Done</info>");
      $commandLock->release();
    }
  }

  private function installNpmPackages($output) {
    $output->writeln("<comment>".date('Y-m-d H:i:s')." - Installing npm packages...</comment>");

    $process = new Process('npm install');
    $process->setTimeout(300);
    $process->run();

    // executes after the command finishes
    if (!$process->isSuccessful()) {
      $output->writeln("<error>".$process->getErrorOutput()."</error>");
    }
    $output->writeln("<comment>".$process->getOutput()."</comment>");
  }

  private function gulpBuild($output) {
    $output->writeln("<comment>".date('Y-m-d H:i:s')." - Running gulp build...</comment>");

    $process = new Process('gulp build');
    $process->setTimeout(300);
    $process->run();

    // executes after the command finishes
    if (!$process->isSuccessful()) {
      $output->writeln("<error>".$process->getErrorOutput()."</error>");
    }
    $output->writeln("<comment>".$process->getOutput()."</comment>");
  }

  private function gulpBowerInstall($output) {
    $output->writeln("<comment>".date('Y-m-d H:i:s')." - Running gulp bower-install...</comment>");

    $process = new Process('gulp bower-install');
    $process->setTimeout(300);
    $process->run();

    // executes after the command finishes
    if (!$process->isSuccessful()) {
      $output->writeln("<error>".$process->getErrorOutput()."</error>");
    }
    $output->writeln("<comment>".$process->getOutput()."</comment>");
  }

}
