<?php

namespace Bazookas\CommonBundle\Entity\Traits;

use Bazookas\CommonBundle\Entity\Interfaces\AccessControlInterface;
use Bazookas\CommonBundle\Security\Roles;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Security\Core\Authorization\AuthorizationChecker;
use Bazookas\CommonBundle\Entity\Base\BaseCMSUser;

trait BaseAccessControlTrait {

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canAdd(AuthorizationChecker $auth, BaseCMSUser $user) {
    return $auth->isGranted($this->getRoleRequiredForAdd());
  }

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canEdit(AuthorizationChecker $auth, BaseCMSUser $user) {
    return $auth->isGranted($this->getRoleRequiredForEdit());
  }

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canDelete(AuthorizationChecker $auth, BaseCMSUser $user) {
    return $auth->isGranted($this->getRoleRequiredForDelete());
  }

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canView(AuthorizationChecker $auth, BaseCMSUser $user) {
    return $auth->isGranted($this->getRoleRequiredForView());
  }

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canExport(AuthorizationChecker $auth, BaseCMSUser $user) {
    return $auth->isGranted($this->getRoleRequiredForView());
  }

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canBulkEdit(AuthorizationChecker $auth, BaseCMSUser $user) {
    return $auth->isGranted($this->getRoleRequiredForView());
  }

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canBulkRemove(AuthorizationChecker $auth, BaseCMSUser $user) {
    return $auth->isGranted($this->getRoleRequiredForView());
  }

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @param string $action
   * @return bool
   */
  public function performAccessCheck(AuthorizationChecker $auth, BaseCMSUser $user, $action) {
    switch($action) {
      case AccessControlInterface::ACTION_ADD:
        return $this->canAdd($auth, $user);
      case AccessControlInterface::ACTION_EDIT:
        return $this->canEdit($auth, $user);
      case AccessControlInterface::ACTION_REMOVE:
        return $this->canDelete($auth, $user);
      case AccessControlInterface::ACTION_VIEW:
        return $this->canView($auth, $user);
      case AccessControlInterface::ACTION_EXPORT:
        return $this->canExport($auth, $user);
      case AccessControlInterface::ACTION_BULK_EDIT:
        return $this->canBulkEdit($auth, $user);
      case AccessControlInterface::ACTION_BULK_REMOVE:
        return $this->canBulkRemove($auth, $user);
      default:
        return false;
    }
  }

  /**
   * @return string
   */
  public function getRoleRequiredForAdd() {
    return $this->getRoleRequiredForView();
  }

  /**
   * @return string
   */
  public function getRoleRequiredForEdit() {
    return $this->getRoleRequiredForView();
  }

  /**
   * @return string
   */
  public function getRoleRequiredForDelete() {
    return $this->getRoleRequiredForView();
  }

  /**
   * @return string
   */
  public function getRoleRequiredForView() {
    return Roles::ROLE_SUPER_ADMIN;
  }

  /**
   * @return string
   */
  public function getRoleRequiredForExport() {
    return $this->getRoleRequiredForView();
  }

}
