<?php

namespace Tests\Bazookas\CommonBundle\Utility;

use Bazookas\CommonBundle\Tests\Bazookas\CommonBundle\Entity\NestedTestEntity;
use Bazookas\CommonBundle\Utility\FormattingUtility;
use Bazookas\CommonBundle\Utility\Utility;
use Doctrine\Common\Collections\ArrayCollection;
use Symfony\Bundle\FrameworkBundle\Test\KernelTestCase;
use Bazookas\CommonBundle\Tests\Bazookas\CommonBundle\Entity\SimpleTestEntity;

class FormattingUtilityTest extends KernelTestCase
{
  private $container;

  public function setUp()
  {
    self::bootKernel();
    $this->container = self::$kernel->getContainer();
  }

  private function createSimpleTestEntity() {
    $entity = new SimpleTestEntity();
    $entity
      ->setPrivateName('name')
      ->setPrivateDescription('description')
      ->setPrivateArrayItem(['a test'])

      ->setProtectedName('name')
      ->setProtectedDescription('description')
      ->setProtectedArrayItem(['a test'])

      ->setPublicName('name')
      ->setPublicDescription('description')
      ->setPublicArrayItem(['a test'])
    ;

    return $entity;
  }

  private function createNestedTestEntity() {
    $entity = new NestedTestEntity();

    $simpleEntity1 = $this->createSimpleTestEntity();

    $entity
      ->setName('test')
      ->setArrayItems([
        $simpleEntity1,
        $this->createSimpleTestEntity()
      ])
     ->setArrayCollectionItems(new ArrayCollection([
       $simpleEntity1,
       $this->createSimpleTestEntity()
     ]))
    ;

    return $entity;
  }

  private function createCircularReferenceTestEntity() {
    $simpleEntity = $this->createSimpleTestEntity();
    $simpleEntity2 = $this->createSimpleTestEntity();
    $simpleEntity->setProtectedArrayItem([$simpleEntity2]);
    $simpleEntity2->setProtectedArrayItem([$simpleEntity]);

    return $simpleEntity;
  }

  /**
   * Test the conversion of a simple entity
   */
  public function testSimpleEntityConversion() {
    $testEntity = $this->createSimpleTestEntity();

    $convertedEntity = FormattingUtility::entityToArray($testEntity);

    //Assert existence of private properties
    $this->assertTrue(isset($convertedEntity['privateName']), 'Failed to assert that private property was converted');
    $this->assertTrue(isset($convertedEntity['privateDescription']), 'Failed to assert that private property was converted');
    $this->assertTrue(isset($convertedEntity['privateArrayItem']), 'Failed to assert that private property was converted');

    //Assert existence of protected properties
    $this->assertTrue(isset($convertedEntity['protectedName']), 'Failed to assert that protected property was converted');
    $this->assertTrue(isset($convertedEntity['protectedDescription']), 'Failed to assert that protected property was converted');
    $this->assertTrue(isset($convertedEntity['protectedArrayItem']), 'Failed to assert that protected property was converted');

    //Assert existence of public properties
    $this->assertTrue(isset($convertedEntity['publicName']), 'Failed to assert that public property was converted');
    $this->assertTrue(isset($convertedEntity['publicDescription']), 'Failed to assert that public property was converted');
    $this->assertTrue(isset($convertedEntity['publicArrayItem']), 'Failed to assert that public property was converted');
  }

  /**
   * Test nested entities
   */
  public function testNestedEntityConversion() {
    $testEntity = $this->createNestedTestEntity();

    $convertedEntity = FormattingUtility::entityToArray($testEntity);

    //Assert properties exist
    $this->assertTrue(isset($convertedEntity['name']), 'Failed to assert that the name field was converted correctly');
    $this->assertTrue(isset($convertedEntity['arrayItems']), 'Failed to assert that the arrayItems were converted correctly');
    $this->assertTrue(isset($convertedEntity['arrayCollectionItems']), 'Failed to assert that the arrayCollectionItems were converted correctly');
  }

  /**
   * Test circular reference entities
   */
  public function testCircularReferenceEntities() {
    $testEntity = $this->createCircularReferenceTestEntity();

    $convertedEntity = FormattingUtility::entityToArray($testEntity);

    $entities = [$convertedEntity, $convertedEntity['protectedArrayItem'][0]];

    foreach($entities as $entity) {
      //Assert existence of private properties
      $this->assertTrue(isset($entity['privateName']), 'Failed to assert that private property was converted');
      $this->assertTrue(isset($entity['privateDescription']), 'Failed to assert that private property was converted');
      $this->assertTrue(isset($entity['privateArrayItem']), 'Failed to assert that private property was converted');

      //Assert existence of protected properties
      $this->assertTrue(isset($entity['protectedName']), 'Failed to assert that protected property was converted');
      $this->assertTrue(isset($entity['protectedDescription']), 'Failed to assert that protected property was converted');
      $this->assertTrue(isset($entity['protectedArrayItem']), 'Failed to assert that protected property was converted');

      //Assert existence of public properties
      $this->assertTrue(isset($entity['publicName']), 'Failed to assert that public property was converted');
      $this->assertTrue(isset($entity['publicDescription']), 'Failed to assert that public property was converted');
      $this->assertTrue(isset($entity['publicArrayItem']), 'Failed to assert that public property was converted');
    }

    $this->assertTrue(Utility::startsWith($convertedEntity['protectedArrayItem'][0]['protectedArrayItem'][0], 'object-hash-'));
  }
}
