<?php

namespace Bazookas\CronBundle\Services\Base;

use Bazookas\CronBundle\Entity\Cron;
use Bazookas\CronBundle\Services\Interfaces\CronServiceInterface;
use Symfony\Bundle\TwigBundle\TwigEngine;
use Symfony\Component\Translation\TranslatorInterface;

class BaseCronService implements CronServiceInterface {

  protected $mailer;
  protected $templateEngine;
  protected $translator;

  public function __construct(\Swift_Mailer $mailer, TwigEngine $templateEngine, TranslatorInterface $translator) {
    $this->mailer = $mailer;
    $this->templateEngine = $templateEngine;
    $this->translator = $translator;
  }

  public function onFailure(Cron $cron, array $options) {
    $to = $options['mailer_to'];
    $subject = $this->getOnFailureSubject($cron, $options);
    $plainText = $this->getOnFailurePlainText($cron, $options);
    $html = $this->getOnFailureTemplate($cron, $options);
    $this->sendMail($to, $subject, $plainText, $html, $options['mailer_from']);
  }

  public function onLocked(Cron $cron, array $options) {
    $to = $options['mailer_to'];
    $subject = $this->getOnLockedSubject($cron, $options);
    $plainText = $this->getOnLockedPlainText($cron, $options);
    $html = $this->getOnLockedTemplate($cron, $options);
    $this->sendMail($to, $subject, $plainText, $html, $options['mailer_from']);
  }

  public function onSuccess(Cron $cron, array $options) {
    // Not (yet) implemented in base version
  }

  protected function getOnFailureTemplate(Cron $cron, $options) {
    return $this->templateEngine->render('BazookasCronBundle:Emails:onFailureMail.html.twig', array(
      'cronCommand' => $cron->getCommand(),
      'failureCount' => $cron->getFailureCount(),
      'baseUrl' => $options['baseUrl'],
      'kernelRootDir' => $options['kernelRootDir'],
    ));
  }

  protected function getOnFailureSubject(Cron $cron, $options) {
    return $this->translator->trans('admin.mail.failure', [],'admin') . ' '. $options['baseUrl'];
  }

  protected function getOnFailurePlainText(Cron $cron, $options) {
    return strip_tags($this->getOnFailureTemplate($cron, $options));
  }

  protected function getOnLockedTemplate(Cron $cron, $options) {
    return $this->templateEngine->render('BazookasCronBundle:Emails:onLockedMail.html.twig', array(
      'cronCommand' => $cron->getCommand(),
      'runTime' => $cron->getNextRunOn()->format('Y-m-d H:i:s'),
      'maxExecutionTime' => $cron->getMaxExecutionTime(),
      'baseUrl' => $options['baseUrl'],
      'kernelRootDir' => $options['kernelRootDir'],
    ));
  }

  protected function getOnLockedSubject(Cron $cron, $options) {
    return $this->translator->trans('admin.mail.locked', [],'admin') . $options['baseUrl'];
  }

  protected function getOnLockedPlainText(Cron $cron, $options) {
    return strip_tags($this->getOnLockedTemplate($cron, $options));
  }

  protected function getOnSuccessTemplate(Cron $cron, $options) {
    // Not (yet) implemented in base version
  }

  protected function getOnSuccessSubject(Cron $cron, $options) {
    // Not (yet) implemented in base version
  }

  protected function getOnSuccessPlainText(Cron $cron, $options) {
    // Not (yet) implemented in base version
  }

  protected function sendMail($to, $subject, $plainText, $html, $from = 'support@bazookas.be') {
    $message =  \Swift_Message::newInstance()
      ->setSubject($subject)
      ->setFrom($from)
      ->setTo($to)
      ->setBody($plainText, 'text/plain')
      ->addPart($html,'text/html')
    ;

    $this->mailer->send($message);
  }
}
