<?php

namespace Bazookas\ExportBundle\Services;

use Symfony\Component\HttpFoundation\StreamedResponse;

class ExportFileCreationService {

  /**
   * @param array $exportData
   * @param array $exportConfig
   * @return StreamedResponse
   */
  public function createCsvFile(array $exportData, array $exportConfig) {
    $exportData = $exportData[0];

    $fileName = $exportConfig['fileName'] . '.' . $exportConfig['format'];

    $response = new StreamedResponse();
    $response->setCallback(function() use ($exportData, $fileName) {
      $handle = fopen('php://output', 'w+');
      fputcsv($handle, $exportData['headers'], ';');
      foreach ($exportData['data'] as $row) {
        fputcsv($handle, $row, ';');
      }
      fclose($handle);
    });

    $response->setStatusCode(200);
    $response->headers->set('Content-Type', 'text/csv; charset=utf-8');
    $response->headers->set('Content-Disposition', 'attachment; filename="'.$fileName.'"');
    return $response;
  }

  /**
   * @param array $exportData
   * @param array $exportConfig
   * @return StreamedResponse
   */
  public function createXlsxFile(array $exportData, array $exportConfig) {
    $fileName = $exportConfig['fileName'] . '.' . $exportConfig['format'];

    $response = new StreamedResponse();
    $response->setCallback(function() use ($exportData, $fileName) {
      \PHPExcel_Settings::setZipClass(\PHPExcel_Settings::PCLZIP);
      $objPHPExcel = new \PHPExcel();
      for ($i = 0; $i < count($exportData); $i++) {
        if ($i !== 0) {
          $objPHPExcel->createSheet();
        }
        $objPHPExcel->setActiveSheetIndex($i);
        $objPHPExcel->getActiveSheet()->fromArray($exportData[$i]['headers'], null, 'A1');
        $objPHPExcel->getActiveSheet()->fromArray($exportData[$i]['data'], null, 'A2');
        if (!isset($exportData[$i]['worksheet'])) {
          $objPHPExcel->getActiveSheet()->setTitle('Sheet ' . ($i + 1));
        } else {
          $objPHPExcel->getActiveSheet()->setTitle($exportData[$i]['worksheet']);
        }
      }
      // Auto size columns for each worksheet
      foreach ($objPHPExcel->getWorksheetIterator() as $worksheet) {

        $objPHPExcel->setActiveSheetIndex($objPHPExcel->getIndex($worksheet));

        $sheet = $objPHPExcel->getActiveSheet();
        $cellIterator = $sheet->getRowIterator()->current()->getCellIterator();
        $cellIterator->setIterateOnlyExistingCells(true);
        foreach ($cellIterator as $cell) {
          $sheet->getColumnDimension($cell->getColumn())->setAutoSize(true);
        }
      }
      $objPHPExcel->setActiveSheetIndex(0);
      $objWriter = \PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
      $objWriter->save('php://output');
    });

    $response->setStatusCode(200);
    $response->headers->set('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
    $response->headers->set('Content-Disposition', 'attachment; filename="'.$fileName.'"');
    $response->headers->set('Cache-Control', 'max-age:0');
    return $response;
  }
}
