<?php
namespace Bazookas\GeneratorBundle\Generator;

use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\HttpKernel\Bundle\BundleInterface;
use Sensio\Bundle\GeneratorBundle\Generator\Generator;
use Sensio\Bundle\GeneratorBundle\Generator\DoctrineFormGenerator;
use Doctrine\ORM\Mapping\ClassMetadataInfo;

/*
 * NOTE I tried to reuse the DoctrineFormGenerator, but it didn't work...
 * the doctrine form generator can't generate forms for entities in other bundle then form class...
 */
class AdminControllerGenerator extends Generator
{
    private $filesystem;
    private $skeletonDir;
    private $className;
    private $classPath;

    /**
     * Constructor.
     *
     * @param Filesystem $filesystem A Filesystem instance
     */
    public function __construct(Filesystem $filesystem)
    {
      $this->skeletonDir = __DIR__ . '/../Resources/SensioGeneratorBundle/skeleton';
      $this->filesystem = $filesystem;
      $this->formGenerator = new DoctrineFormGenerator($filesystem);
    }

    public function generate(BundleInterface $bundle, $controller, $routeFormat, $route, $entity, $metadata)
    {
      // first generate a form class
      $this->generateRouting($bundle, $controller, $route, $routeFormat);
      $this->generateForm($bundle, $entity, $metadata);
      $this->generateController(
        $bundle,
        $controller,
        $routeFormat,
        "{$bundle->getNamespace()}\\Form\\{$this->className}",
        $entity,
        $metadata);

    }

    public function generateController(BundleInterface $bundle, $controller, $routeFormat, $form, $entity, $metadata)
    {
      $parts = explode('\\', $entity);
      $entityClass = array_pop($parts);

      $parts = explode('\\', $form);
      $formClass = array_pop($parts);

      $dir = $bundle->getPath();
      $controllerFile = $dir.'/Controller/'.$controller.'Controller.php';
      if (file_exists($controllerFile)) {
          throw new \RuntimeException(sprintf('Controller "%s" already exists', $controller));
      }

      $parameters = array(
          'entityName' => lcfirst($entityClass),
          'entityUse' => $metadata->name,
          'entityClass' => $entityClass,
          'fields' => $this->getFieldsFromMetadata($metadata),
          'namespace' => $bundle->getNamespace(),
          'bundle' => $bundle->getName(),
          'format' => array(
              'routing' => $routeFormat,
          ),
          'formUse' => $form,
          'formClass' => $formClass,
          'controller' => $controller,
      );

      $this->setSkeletonDirs(array(
        $this->skeletonDir
      ));
      $this->renderFile('controller/AdminController.php.twig', $controllerFile, $parameters);
    }


    /**
     * Generates the entity form class.
     *
     * @param BundleInterface   $bundle         The bundle in which to create the class
     * @param string            $entity         The entity relative class name
     * @param ClassMetadataInfo $metadata       The entity metadata class
     * @param bool              $forceOverwrite If true, remove any existing form class before generating it again
     */
    public function generateForm(BundleInterface $bundle, $entity, ClassMetadataInfo $metadata, $forceOverwrite = false)
    {
        $parts = explode('\\', $entity);
        $entityClass = array_pop($parts);

        $this->className = $entityClass.'AdminType';
        $dirPath = $bundle->getPath().'/Form';
        $this->classPath = $dirPath.'/'.str_replace('\\', '/', $entity).'AdminType.php';

        if (!$forceOverwrite && file_exists($this->classPath)) {
            throw new \RuntimeException(sprintf('Unable to generate the %s form class as it already exists under the %s file', $this->className, $this->classPath));
        }

        if (count($metadata->identifier) > 1) {
            throw new \RuntimeException('The form generator does not support entity classes with multiple primary keys.');
        }

        $parts = explode('\\', $entity);
        array_pop($parts);

        $this->setSkeletonDirs(array(
          $this->skeletonDir
        ));
        $this->renderFile('form/FormType.php.twig', $this->classPath, array(
            'entityName' => lcfirst($entityClass),
            'entityUse' => $metadata->name,
            'entityClass' => $entityClass,
            'fields' => $this->getFieldsFromMetadata($metadata),
            'fields_mapping' => $metadata->fieldMappings,
            'namespace' => $bundle->getNamespace(),
            'form_namespace' => implode('\\', $parts),
            'bundle' => $bundle->getName(),
            'form_class' => $this->className,
            'form_type_name' => strtolower(str_replace('\\', '_', $bundle->getNamespace()).($parts ? '_' : '').implode('_', $parts).'_'.substr($this->className, 0, -4)),

            // Add 'setDefaultOptions' method with deprecated type hint, if the new 'configureOptions' isn't available.
            // Required as long as Symfony 2.6 is supported.
            'configure_options_available' => method_exists('Symfony\Component\Form\AbstractType', 'configureOptions'),
            'get_name_required' => !method_exists('Symfony\Component\Form\AbstractType', 'getBlockPrefix'),
        ));
    }

    public function generateRouting(BundleInterface $bundle, $controller, $route, $format)
    {
        // annotation is generated in the templates
        if ('annotation' == $format) {
            return true;
        }

        $file = $bundle->getPath().'/Resources/config/routing.'.$format;
        if (file_exists($file)) {
            $content = file_get_contents($file);
        } elseif (!is_dir($dir = $bundle->getPath().'/Resources/config')) {
            mkdir($dir);
        }

        $name = strtolower(preg_replace(
          array('/^([A-Z])/','/(Bundle)/','/(.)([A-Z])/'),
          array('$1','','$1_$2'),
          $bundle->getName().$controller)
        );
        // $name = preg_replace('/^(_)/', '', $name);
        $controller = $bundle->getName().':'.$controller.':index';

        if ('yml' == $format) {
            // yaml
            if (!isset($content)) {
                $content = '';
            }

            $content .= sprintf(
                "\n%s:".
                "\n    path:     /{_locale}/%s/{_action}/{_id}".
                "\n    defaults:".
                "\n      _controller: %s".
                "\n      _action: ~".
                "\n      _id: ~".
                "\n",
                $name,
                $route,
                $controller
            );
        } elseif ('xml' == $format) {
          throw new \InvalidArgumentException('xml routing format not yet supported');
        } elseif ('php' == $format) {
          throw new \InvalidArgumentException('php routing format not yet supported');
        }

        $flink = fopen($file, 'w');
        if ($flink) {
            $write = fwrite($flink, $content);

            if ($write) {
                fclose($flink);
            } else {
                throw new \RunTimeException(sprintf('We cannot write into file "%s", has that file the correct access level?', $file));
            }
        } else {
            throw new \RunTimeException(sprintf('Problems with generating file "%s", did you gave write access to that directory?', $file));
        }
    }


    /**
     * Returns an array of fields. Fields can be both column fields and
     * association fields.
     *
     * @param ClassMetadataInfo $metadata
     *
     * @return array $fields
     */
    private function getFieldsFromMetadata(ClassMetadataInfo $metadata)
    {
        $fields = (array) $metadata->fieldNames;

        // Remove the primary key field if it's not managed manually
        if (!$metadata->isIdentifierNatural()) {
            $fields = array_diff($fields, $metadata->identifier);
        }

        foreach ($metadata->associationMappings as $fieldName => $relation) {
            if ($relation['type'] !== ClassMetadataInfo::ONE_TO_MANY) {
                $fields[] = $fieldName;
            }
        }

        return $fields;
    }
}
