<?php

namespace Bazookas\IntegrationsBundle\DependencyInjection;

use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\HttpKernel\DependencyInjection\Extension;
use Symfony\Component\DependencyInjection\Loader;
use Symfony\Component\DependencyInjection\Reference;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
/**
 * This is the class that loads and manages your bundle configuration.
 *
 * @link http://symfony.com/doc/current/cookbook/bundles/extension.html
 */
class BazookasIntegrationsExtension extends Extension implements CompilerPassInterface
{
  /**
   * {@inheritdoc}
   */
  public function load(array $configs, ContainerBuilder $container)
  {
      $configuration = new Configuration();
      $config = $this->processConfiguration($configuration, $configs);

      $loader = new Loader\YamlFileLoader($container, new FileLocator(__DIR__.'/../Resources/config'));
      $loader->load('services.yml');

      if (!empty($config)) {
        foreach($config as $integrationKey => $integrationSettings) {
          switch($integrationKey) {
            case "facebook":
              $this->configureFacebookIntegration($container, $integrationSettings);
              break;
            default:
              throw new \Exception('Unkown integration \''.$integrationKey.'\' ');
          }
        }
      }
  }

  private function configureFacebookIntegration(ContainerBuilder $container, $config) {
    // set some defaults
    if (empty($config['fields'])) {
      $config['fields'] = array(
        'email' => 'setEmail'
      );
    }

    // set them as parameters
    foreach($config as $key => $value) {
      $container->setParameter('bazookas_integrations.facebook.'.$key, $value);
    }

    //see http://symfony.com/doc/current/components/dependency_injection.html
    //Enable the facebook service
    $container
      ->register('BazookasIntegrationsBundle.services.FacebookService', 'Bazookas\IntegrationsBundle\Services\Facebook\FacebookService')
      ->setLazy(true)
      ->addArgument($config)
    ;

  }

  public function process(ContainerBuilder $container)
  {
    if ($container->has('BazookasIntegrationsBundle.services.FacebookService') && $container->has('bazookas.common.exceptionManager')) {
      $facebookService = $container->getDefinition('BazookasIntegrationsBundle.services.FacebookService');
      $facebookService->addMethodCall('setExceptionManager', array(new Reference('bazookas.common.exceptionManager')));
    }
  }

}
