# Facebook

## Activation

Activate the facebook integration by providing the following config.

### Configuration in yml

```yml
bazookas_api_framework:
  oAuth:
    facebookGrantTypeEnabled: true
    facebookGrantTypeUserProvider: APIBundle.user_provider.BaseUserProvider

bazookas_integrations:
  facebook:
    # credentials are optional, but recomended when working with different credentials in one project
    # leave this blank and use "setCredentials" method on facebookservice
    appId: "%facebook_app_id%"
    appSecret: "%facebook_app_secret%"
    
    # default it is assumed the facebook-id is stored in the 'id' field of the user entity.
    # If this is not the case, a different field can be provided like in this config
    facebookIdField: facebookId
    
    # which fields to fetch from the facebook graph API and where to put them in the user entity
    # when value is a string, it will be interpreted as a setter method
    fields:
      first_name: setFirstName
      last_name: setLastName
      gender: setGender
      email: setEmail
      # otherwhise, for speciel fields like the picture,
      # you need to define the "setter method" as a property
      picture:
        width: 100
        height: 100
        method: setImage
        imageClass: Bazookas\MediaBundle\Entity\Image
```

Add a user provider as service (and point to it as in config above)
```
  APIBundle.user_provider.BaseUserProvider:
    class: Bazookas\SecurityBundle\UserProviders\BaseUserProvider
    arguments:
      - "@doctrine.orm.entity_manager"
      - { entity: 'APIBundle\Entity\AppUser' }
```

## Usage

### Services

- FacebookService
  - handles communication between app and facebook graph api
  - requests made without access token will fallback to an app access token
  - this way you can get around expired user access tokens
- FacebookOAuthGrantExtension
  - adds a grant type to fos oauth server
  - will create users when not yet registered and `bazookas_api_framework.oAuth.allowRegistration` is true

### Setting up your AppUser entity

The user entity should implement `Bazookas\IntegrationsBundle\Entity\Interfaces\FacebookUserInterface`.

```php
/**
 * @ORM\Column(type="string")
 */
private $facebookAccessToken;

public function getFacebookAccessToken()
{
  return $this->facebookAccessToken;
}

public function setFacebookAccessToken($facebookAccessToken)
{
  $this->facebookAccessToken = $facebookAccessToken;
  return $this;
}
```
