<?php

namespace Bazookas\MediaBundle\DependencyInjection;

use Bazookas\CommonBundle\DependencyInjection\Loader\PrependYamlLoader;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\DependencyInjection\Extension\Extension;
use Symfony\Component\DependencyInjection\Extension\PrependExtensionInterface;
use Symfony\Component\DependencyInjection\Loader;

/**
 * This is the class that loads and manages your bundle configuration.
 *
 * @link http://symfony.com/doc/current/cookbook/bundles/extension.html
 */
class BazookasMediaExtension extends Extension implements PrependExtensionInterface
{
  /**
   * {@inheritdoc}
   */
  public function load(array $configs, ContainerBuilder $container)
  {
    $configuration = new Configuration();
    $config = $this->processConfiguration($configuration, $configs);

    $loader = new Loader\YamlFileLoader($container, new FileLocator(__DIR__.'/../Resources/config'));
    $loader->load('services.yml');

    $container->setParameter('bazookas_media.settings', $config['settings']);
    $container->setParameter('bazookas_media.video_providers', $config['video_providers']);
    $container->setParameter('bazookas_media.default_folders', $config['default_folders']);

    $this->addProviderServices($container, $config['video_providers']);
  }

  public function addProviderServices(ContainerBuilder $container, $providers) {
    foreach($providers as $provider) {
      switch($provider) {
        case 'vimeo':
          $this->addVimeoService($container, $provider);
          break;
      }
    }
  }

  public function prepend(ContainerBuilder $container)
  {
    $loader = new PrependYamlLoader($container, new FileLocator(__DIR__.'/../Resources/config'));
    $loader->load('config.yml');
  }

  private function addVimeoService(ContainerBuilder $container, $provider) {
    $container
      ->register('BazookasMediaBundle.media.VimeoService', 'Bazookas\MediaBundle\Services\Vimeo\VimeoService')
      ->addArgument("%bazookas_media.video_providers%")
      ->addTag('BazookasMediaBundle.media.VideoService')
    ;
  }

}
