<?php

namespace Bazookas\MediaBundle\Form;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Bazookas\MediaBundle\Form\BaseMediaType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Bazookas\MediaBundle\Entity\Video;

class VideoProviderType extends AbstractType
{
  private $videoProviders;

  public function __construct($videoProviders) {
    $this->videoProviders = $videoProviders;
  }

  /**
  * @param FormBuilderInterface $builder
  * @param array $options
  */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('accessToken', TextType::class, array(
        'label' => 'media.entities.videoProvider.fields.accessToken',
      ))
    ;

    $providerChoices = $this->videoProvidersAsChoiceArray($this->videoProviders);

    if (count($providerChoices) > 1) {
      $builder
        ->add('provider', ChoiceType::class, array(
          'label' => 'media.entities.video.fields.provider',
          'choices' => $providerChoices
        ))
      ;
    } else {
      $entity = $builder->getData();
      $entity->setProvider(reset($providerChoices));
    }
  }

  /**
  * @param OptionsResolver $resolver
  */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults(array(
      'data_class' => 'Bazookas\MediaBundle\Entity\VideoProvider'
    ));
  }

  private function videoProvidersAsChoiceArray($videoProviders) {
    $reflClass = new \ReflectionClass(Video::class);

    $providerChoices = [];
    foreach($videoProviders as $provider) {
      $providerChoices['media.entities.video.enums.'.$provider['type']] = $reflClass->getConstant($provider['type']);
    }

    return $providerChoices;
  }

}
