<?php

namespace Bazookas\MediaBundle\Repository\Base;

use Bazookas\CommonBundle\Repository\Base\BaseRepository;
use Bazookas\MediaBundle\Form\BaseMediaType;
use Bazookas\MediaBundle\Entity\Base\BaseMedia;
use Bazookas\MediaBundle\Entity\MediaFolder;

class BaseMediaRepository extends BaseRepository{

  public function getEntityClass() {
    return BaseMedia::class;
  }

  public function getMediaFolders($parent, $includeHidden = false) {
    $qb = $this->entityManager->createQueryBuilder();
    $qb
      ->select('f')
      ->from(MediaFolder::class, 'f')
      ->addOrderBy('f.name', 'ASC')
    ;

    if (!empty($parent)) {
      $qb
        ->andWhere($qb->expr()->eq('f.parent', ':parent'))
        ->setParameter('parent', $parent)
      ;
    } else {
      $qb
        ->andWhere($qb->expr()->isNull('f.parent'))
      ;
    }

    if (!$includeHidden) {
      $qb
        ->andWhere($qb->expr()->neq('f.type', ':fType') )
        ->setParameter('fType', MediaFolder::TYPE_HIDDEN_FOLDER)
      ;
    }

    $folders = $qb->getQuery()->getArrayResult();

    //get the parent folder so we can create an "up" button
    if (!empty($parent)) {
      $qb = $this->entityManager->createQueryBuilder();
      $qb
        ->select('f', 'fp')
        ->from(MediaFolder::class, 'f')
        ->andWhere($qb->expr()->eq('f.id', ':id'))
        ->leftJoin('f.parent', 'fp')
        ->setParameter('id', $parent)
      ;

      $parentFolder = $qb->getQuery()->getArrayResult()[0];
      $parentFolder['isParentBlock'] = true;
      array_unshift($folders, $parentFolder);
    }

    return $folders;
  }

  public function getMediaFoldersAsArray($includeHidden = false) {
    $qb = $this->entityManager->createQueryBuilder();
    $qb
      ->select('f, partial fp.{id, type}')
      ->from(MediaFolder::class, 'f')
      ->leftJoin('f.parent', 'fp')
    ;

    if (!$includeHidden) {
      $qb
        ->andWhere($qb->expr()->neq('f.type', ':fType') )
        ->setParameter('fType', MediaFolder::TYPE_HIDDEN_FOLDER)
      ;
    }

    return $qb->getQuery()->getArrayResult();
  }

  public function getFolderBreadCrumb($folderId) {
    $breadcrumb = array();
    if ($folderId) {
      $qb = $this->entityManager->createQueryBuilder();
      $qb
        ->select('f')
        ->from(MediaFolder::class, 'f')
        ->andWhere($qb->expr()->eq('f.id', ':id'))
        ->setParameter('id', $folderId)
      ;
      $query = $qb->getQuery();
      $query->setFetchMode(MediaFolder::class, 'parent', \Doctrine\ORM\Mapping\ClassMetadata::FETCH_EAGER);
      $folder = $query->getOneOrNullResult();

      $this->unshiftParent($folder, $breadcrumb);
    }
    array_unshift($breadcrumb, array(
      'label' => 'media.breadcrumb.root',
      'id' => null
    ));
    return $breadcrumb;
  }

  private function unshiftParent($parent, &$breadcrumb) {
    if ($parent) {
      array_unshift($breadcrumb, array(
        'label' => $parent->getName(),
        'id' => $parent->getId()
      ));
      $this->unshiftParent($parent->getParent(), $breadcrumb);
    }
  }

  public function getMediaFolderById($folderId) {
    $qb = $this->entityManager->createQueryBuilder();
    $qb
      ->select('f')
      ->from(MediaFolder::class, 'f')
      ->andWhere($qb->expr()->eq('f.id', ':id'))
      ->setParameter('id', $folderId)
    ;
    return $qb->getQuery()->getOneOrNullResult();
  }

  public function getMediaFolderByDefaultId($defaultId) {
    $qb = $this->entityManager->createQueryBuilder();
    $qb
      ->select('f')
      ->from(MediaFolder::class, 'f')
      ->andWhere($qb->expr()->eq('f.defaultFolder', ':defaultId'))
      ->setParameter('defaultId', $defaultId)
    ;

    return $qb->getQuery()->getOneOrNullResult();
  }

  /**
   * TODO deprecated? check if this function is used in any project and remove this
   * @param $id
   * @param $path
   * @param $relativeUrl
   * @param $url
   * @return array
   */
  public function updateMediaFilePath($id, $path, $relativeUrl, $url) {
    $result = $this->entityManager
      ->createQuery('
        UPDATE Bazookas\MediaBundle\Entity\Base\BaseMedia m
        SET m.url = :url, m.relativeUrl = :relativeUrl, m.path = :path
        WHERE m.id = :id 
      ')
      ->setParameters(array(
        'id' => $id,
        'url' => $url,
        'relativeUrl' => $relativeUrl,
        'path' => $path,
      ))
      ->getResult()
    ;

    return $result;
  }
}
